@@ -1,6 +1,6 @@
 /*
  *  git2r, R bindings to the libgit2 library.
- *  Copyright (C) 2013-2016 The git2r contributors
+ *  Copyright (C) 2013-2018 The git2r contributors
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -21,11 +21,12 @@
 
 #include <R.h>
 #include <Rinternals.h>
-#include "git2.h"
+#include <git2.h>
 
 int git2r_arg_check_blob(SEXP arg);
 int git2r_arg_check_branch(SEXP arg);
 int git2r_arg_check_commit(SEXP arg);
+int git2r_arg_check_commit_stash(SEXP arg);
 int git2r_arg_check_credentials(SEXP arg);
 int git2r_arg_check_fetch_heads(SEXP arg);
 int git2r_arg_check_filename(SEXP arg);
@@ -35,6 +36,8 @@ int git2r_arg_check_integer_gte_zero(SEXP arg);
 int git2r_arg_check_list(SEXP arg);
 int git2r_arg_check_logical(SEXP arg);
 int git2r_arg_check_note(SEXP arg);
+int git2r_arg_check_repository(SEXP arg);
+int git2r_arg_check_same_repo(SEXP arg1, SEXP arg2);
 int git2r_arg_check_signature(SEXP arg);
 int git2r_arg_check_string(SEXP arg);
 int git2r_arg_check_string_vec(SEXP arg);
