@@ -1,5 +1,5 @@
 ## git2r, R bindings to the libgit2 library.
-## Copyright (C) 2013-2015 The git2r contributors
+## Copyright (C) 2013-2018 The git2r contributors
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License, version 2,
@@ -14,7 +14,7 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-library(git2r)
+library("git2r")
 
 ## For debugging
 sessionInfo()
@@ -70,8 +70,8 @@ push(repo_2, force = TRUE)
 
 ## Check commits
 stopifnot(identical(length(commits(bare_repo)), 2L))
-stopifnot(identical(commits(repo_2)[[1]]@sha, commits(bare_repo)[[1]]@sha))
-stopifnot(identical(commits(repo_2)[[2]]@sha, commits(bare_repo)[[2]]@sha))
+stopifnot(identical(sha(commits(repo_2)[[1]]), sha(commits(bare_repo)[[1]])))
+stopifnot(identical(sha(commits(repo_2)[[2]]), sha(commits(bare_repo)[[2]])))
 
 ## Cleanup
 unlink(path_bare, recursive=TRUE)
